@@ -38,14 +38,6 @@
  * \file
  */
 
-#ifndef _WIN32
-	#include <sys/param.h>
-#else
-	#if defined(_MSC_VER) && _MSC_VER < 1900
-		#define snprintf _snprintf
-	#endif
-#endif /* _WIN32 */
-
 #include "config.h"
 
 #if EXCLUDE_SSDP == 0
@@ -62,6 +54,8 @@
 
 	#include <stdio.h>
 
+	#include "posix_overwrites.h"
+
 	#define MAX_TIME_TOREAD 45
 
 	#ifdef INCLUDE_CLIENT_APIS
@@ -69,7 +63,7 @@ SOCKET gSsdpReqSocket4 = INVALID_SOCKET;
 		#ifdef UPNP_ENABLE_IPV6
 SOCKET gSsdpReqSocket6 = INVALID_SOCKET;
 		#endif /* UPNP_ENABLE_IPV6 */
-	#endif         /* INCLUDE_CLIENT_APIS */
+	#endif	       /* INCLUDE_CLIENT_APIS */
 
 void RequestHandler();
 
@@ -988,9 +982,26 @@ static int create_ssdp_sock_v4(
 		ret = UPNP_E_SOCKET_BIND;
 		goto error_handler;
 	}
-	memset((void *)&ssdpMcastAddr, 0, sizeof(struct ip_mreq));
+	/*
+	 * See: https://man7.org/linux/man-pages/man7/ip.7.html
+	 * Socket options, IP_ADD_MEMBERSHIP
+	 *
+	 * This memset actually sets imr_address to INADDR_ANY and
+	 * imr_ifindex to zero, which make the system choose the appropriate
+	 * interface.
+	 *
+	 * Still using "struct ip_mreq" instead of "struct ip_mreqn" because
+	 * windows does not recognize the latter.
+	 */
+	memset((void *)&ssdpMcastAddr, 0, sizeof ssdpMcastAddr);
+	#ifdef _WIN32
+	inet_pton(AF_INET, (PCSTR)gIF_IPV4, &ssdpMcastAddr.imr_interface);
+	inet_pton(AF_INET, (PCSTR)SSDP_IP, &ssdpMcastAddr.imr_multiaddr);
+	#else
 	ssdpMcastAddr.imr_interface.s_addr = inet_addr(gIF_IPV4);
+	/* ssdpMcastAddr.imr_address.s_addr = inet_addr(gIF_IPV4); */
 	ssdpMcastAddr.imr_multiaddr.s_addr = inet_addr(SSDP_IP);
+	#endif
 	ret = setsockopt(*ssdpSock,
 		IPPROTO_IP,
 		IP_ADD_MEMBERSHIP,
@@ -1010,7 +1021,11 @@ static int create_ssdp_sock_v4(
 	}
 	/* Set multicast interface. */
 	memset((void *)&addr, 0, sizeof(struct in_addr));
+	#ifdef _WIN32
+	inet_pton(AF_INET, (PCSTR)gIF_IPV4, &addr);
+	#else
 	addr.s_addr = inet_addr(gIF_IPV4);
+	#endif
 	ret = setsockopt(*ssdpSock,
 		IPPROTO_IP,
 		IP_MULTICAST_IF,
@@ -1458,7 +1473,7 @@ static int create_ssdp_sock_reqv6(
 	return UPNP_E_SUCCESS;
 }
 		#endif /* IPv6 */
-	#endif         /* INCLUDE_CLIENT_APIS */
+	#endif	       /* INCLUDE_CLIENT_APIS */
 
 int get_ssdp_sockets(MiniServerSockArray *out)
 {
@@ -1489,7 +1504,7 @@ int get_ssdp_sockets(MiniServerSockArray *out)
 	} else
 		out->ssdpReqSock6 = INVALID_SOCKET;
 		#endif /* IPv6 */
-	#endif         /* INCLUDE_CLIENT_APIS */
+	#endif	       /* INCLUDE_CLIENT_APIS */
 	/* Create the IPv4 socket for SSDP */
 	if (strlen(gIF_IPV4) > (size_t)0) {
 		retVal = create_ssdp_sock_v4(&out->ssdpSock4);
